// for UI usage only...
module.exports = {
  db: 'knex',
  conn: 'knex1',
  name: 'books',
  create: true,
  update: true,
  delete: 0,
  import: true,
  export: true,
  multiSelect: true, // multiple selection
  cols: {
    id: { // TBD click to get to pages table
      label: 'ID',
      auto: 'pk',
      add: 'hide',
      edit: 'readonly',
      link: {
        table: 'pages',
        tableId: 'bookId',
        linkId: 'id', //  this column
        path: '/sql-crud' // ui route
      }
    },
    name: {
      label: 'Name',
      type: 'string',
      filter: true,
      default: '',
      ui: {
        tag: 'input'
      },
    },
    rating: {
      label: 'Rating',
      type: 'integer',
      filter: true,
      ui: {
        tag: 'input',
        attrs: { type: 'number', min: 10, max: 90, step: 1 }
      },
      rules: {
        min: 10, max: 90
      }
    },
    yearPublished: {
      label: 'Year Published',
      type: 'string',
      filter: true,
      default: '',
      ui: {
        tag: 'input',
        attrs: { pattern: '^[1-9]\\d{3}$' } // escape twice
      },
      rules: {
        regex: '^[1-9]\\d{3}$'
      }
    },
    categoryId: {
      label: 'Category', // need to check on auto complete
      type: 'integer', // string
      filter: true,
      options: {
        parentCol: '', // use column to get parent value affecting a child
        parentTableColName: '', // the column name in the parent table
        childCol: '', // affect child column in this table
        dbName: 'knex',
        conn: 'knex1',
        tableName: 'categories',
        limit: 8,
        strict: true, // cannot enter own values, must be selected
        key: 'id',
        text: 'name'
      },
      ui: {
        tag: 'bwc-combobox',
        valueType: 'text', //  either key or text
        writeType: 'key', //  either key or text
      }
    },
    // IMPORTANT NOTE: use foreign keys to maintain referential integrity of dependent fields 
    authors: { //  this one data is from join table
      label: 'Authors',
      type: 'string',
      filter: true,
      add: 'hide', // do not allow join tables for create operation
      options: {
        parentCol: '',
        parentTableColName: '',
        childCol: '',
        dbName: 'knex',
        conn: 'knex1',
        tableName: 'authors',
        limit: 8,
        strict: true,
        key: 'id',
        text: 'name'
      },
      ui: {
        tag: 'bwc-combobox',
        junction: {
          link: 'books_authors',
          t1: 'books',
          t2: 'authors',
          t1id: 'id',
          t2id: 'id',
          t1txt: 'name',
          t2txt: 'name',
          refT1id: 'bookId',
          refT2id: 'authorId',  
        },
        attrs: {
          multiple: true,
          tagLimit: 5
        }
      }
    },
    created_at: {
      label: 'Created At',
      auto: 'ts', // autogenerated field... user = user id, ts = timestamp, pk = autogenerated primary key
      hide: true,
      add: 'hide',
      edit: 'readonly'
    },
    updated_at: {
      label: 'Updated At',
      auto: 'ts',
      hide: true,
      add: 'hide',
      edit: 'readonly'
    }
  },

  // generated values
  pk: '',
  multiKey: [],
  required: [],
  auto: [],
  nonAuto: []
  // {id: 1, name: 'book1', categoryId: 1, rating: 5, yearPublished: '2004', created_at: mkDt() },
}
